@@ -27,6 +27,7 @@
 #include "gstvaapiminiobject.h"
 #include "gstvaapidisplay_priv.h"
 #include "gstvaapisurface_priv.h"
+#include "gstvaapiutils_core.h"
 
 #if USE_VA_VPP
 # include <va/va_vpp.h>
@@ -191,9 +192,12 @@ vpp_get_filters_unlocked (GstVaapiFilter * filter, guint * num_filters_ptr)
   *num_filters_ptr = num_filters;
   return filters;
 
+  /* ERRORS */
 error:
-  g_free (filters);
-  return NULL;
+  {
+    g_free (filters);
+    return NULL;
+  }
 }
 
 static VAProcFilterType *
@@ -238,9 +242,12 @@ vpp_get_filter_caps_unlocked (GstVaapiFilter * filter, VAProcFilterType type,
   *num_caps_ptr = num_caps;
   return caps;
 
+  /* ERRORS */
 error:
-  g_free (caps);
-  return NULL;
+  {
+    g_free (caps);
+    return NULL;
+  }
 }
 
 static gpointer
@@ -495,9 +502,12 @@ op_data_new (GstVaapiFilterOp op, GParamSpec * pspec)
   }
   return op_data;
 
+  /* ERRORS */
 error:
-  op_data_free (op_data);
-  return NULL;
+  {
+    op_data_free (op_data);
+    return NULL;
+  }
 }
 
 static inline gpointer
@@ -608,9 +618,12 @@ get_operations_default (void)
   }
   return ops;
 
+  /* ERRORS */
 error:
-  g_ptr_array_unref (ops);
-  return NULL;
+  {
+    g_ptr_array_unref (ops);
+    return NULL;
+  }
 }
 
 /* Get the ordered list of operations, based on VA/VPP queries */
@@ -675,12 +688,15 @@ get_operations_ordered (GstVaapiFilter * filter, GPtrArray * default_ops)
   g_ptr_array_unref (default_ops);
   return ops;
 
+  /* ERRORS */
 error:
-  g_free (filter_caps);
-  g_free (filters);
-  g_ptr_array_unref (ops);
-  g_ptr_array_unref (default_ops);
-  return NULL;
+  {
+    g_free (filter_caps);
+    g_free (filters);
+    g_ptr_array_unref (ops);
+    g_ptr_array_unref (default_ops);
+    return NULL;
+  }
 }
 #endif
 
@@ -977,60 +993,12 @@ deint_refs_clear_all (GstVaapiFilter * filter)
 static gboolean
 ensure_formats (GstVaapiFilter * filter)
 {
-#if VA_CHECK_VERSION(0,34,0)
-  VASurfaceAttrib *surface_attribs = NULL;
-  guint i, num_surface_attribs = 0;
-  VAStatus va_status;
-
   if (G_LIKELY (filter->formats))
     return TRUE;
 
-  GST_VAAPI_DISPLAY_LOCK (filter->display);
-  va_status = vaQuerySurfaceAttributes (filter->va_display, filter->va_config,
-      NULL, &num_surface_attribs);
-  GST_VAAPI_DISPLAY_UNLOCK (filter->display);
-  if (!vaapi_check_status (va_status, "vaQuerySurfaceAttributes()"))
-    return FALSE;
-
-  surface_attribs = g_malloc (num_surface_attribs * sizeof (*surface_attribs));
-  if (!surface_attribs)
-    return FALSE;
-
-  GST_VAAPI_DISPLAY_LOCK (filter->display);
-  va_status = vaQuerySurfaceAttributes (filter->va_display, filter->va_config,
-      surface_attribs, &num_surface_attribs);
-  GST_VAAPI_DISPLAY_UNLOCK (filter->display);
-  if (!vaapi_check_status (va_status, "vaQuerySurfaceAttributes()"))
-    return FALSE;
-
-  filter->formats = g_array_sized_new (FALSE, FALSE, sizeof (GstVideoFormat),
-      num_surface_attribs);
-  if (!filter->formats)
-    goto error;
-
-  for (i = 0; i < num_surface_attribs; i++) {
-    const VASurfaceAttrib *const surface_attrib = &surface_attribs[i];
-    GstVideoFormat format;
-
-    if (surface_attrib->type != VASurfaceAttribPixelFormat)
-      continue;
-    if (!(surface_attrib->flags & VA_SURFACE_ATTRIB_SETTABLE))
-      continue;
-
-    format =
-        gst_vaapi_video_format_from_va_fourcc (surface_attrib->value.value.i);
-    if (format == GST_VIDEO_FORMAT_UNKNOWN)
-      continue;
-    g_array_append_val (filter->formats, format);
-  }
-
-  g_free (surface_attribs);
-  return TRUE;
-
-error:
-  g_free (surface_attribs);
-#endif
-  return FALSE;
+  filter->formats = gst_vaapi_get_surface_formats (filter->display,
+      filter->va_config);
+  return (filter->formats != NULL);
 }
 
 static inline gboolean
@@ -1175,9 +1143,12 @@ gst_vaapi_filter_new (GstVaapiDisplay * display)
     goto error;
   return filter;
 
+  /* ERRORS */
 error:
-  gst_vaapi_filter_unref (filter);
-  return NULL;
+  {
+    gst_vaapi_filter_unref (filter);
+    return NULL;
+  }
 #else
   GST_WARNING ("video processing is not supported, "
       "please consider an upgrade to VA-API >= 0.34");
@@ -1528,10 +1499,13 @@ gst_vaapi_filter_process_unlocked (GstVaapiFilter * filter,
   vaapi_destroy_buffer (filter->va_display, &pipeline_param_buf_id);
   return GST_VAAPI_FILTER_STATUS_SUCCESS;
 
+  /* ERRORS */
 error:
-  deint_refs_clear_all (filter);
-  vaapi_destroy_buffer (filter->va_display, &pipeline_param_buf_id);
-  return GST_VAAPI_FILTER_STATUS_ERROR_OPERATION_FAILED;
+  {
+    deint_refs_clear_all (filter);
+    vaapi_destroy_buffer (filter->va_display, &pipeline_param_buf_id);
+    return GST_VAAPI_FILTER_STATUS_ERROR_OPERATION_FAILED;
+  }
 #endif
   return GST_VAAPI_FILTER_STATUS_ERROR_UNSUPPORTED_OPERATION;
 }
