@@ -49,7 +49,11 @@
 #define GST_VAAPI_DECODE_FLOW_PARSE_DATA        GST_FLOW_CUSTOM_SUCCESS_2
 
 GST_DEBUG_CATEGORY_STATIC (gst_debug_vaapidecode);
+#ifndef GST_DISABLE_GST_DEBUG
 #define GST_CAT_DEFAULT gst_debug_vaapidecode
+#else
+#define GST_CAT_DEFAULT NULL
+#endif
 
 #define GST_VAAPI_DECODE_PARAMS_QDATA \
   g_quark_from_static_string("vaapidec-params")
@@ -65,7 +69,7 @@ static const char gst_vaapidecode_sink_caps_str[] =
     GST_CAPS_CODEC("video/x-xvid")
     GST_CAPS_CODEC("video/x-h263")
     GST_CAPS_CODEC("video/x-h264")
-#if USE_HEVC_DECODER
+#if USE_H265_DECODER
     GST_CAPS_CODEC("video/x-h265")
 #endif
     GST_CAPS_CODEC("video/x-wmv")
@@ -119,7 +123,7 @@ static const GstVaapiDecoderMap vaapi_decode_map[] = {
 #if USE_VP9_DECODER
   {GST_VAAPI_CODEC_VP9, GST_RANK_PRIMARY, "vp9", "video/x-vp9"},
 #endif
-#if USE_HEVC_DECODER
+#if USE_H265_DECODER
   {GST_VAAPI_CODEC_H265, GST_RANK_PRIMARY, "h265", "video/x-h265"},
 #endif
   {0 /* the rest */ , GST_RANK_PRIMARY + 1, NULL,
@@ -134,7 +138,8 @@ static gboolean gst_vaapidecode_update_sink_caps (GstVaapiDecode * decode,
 static gboolean gst_vaapi_decode_input_state_replace (GstVaapiDecode * decode,
     const GstVideoCodecState * new_state);
 
-/* get invoked only if actural VASurface size (not the cropped values) changed */
+/* invoked if actual VASurface size (not the cropped values)
+ * changed */
 static void
 gst_vaapi_decoder_state_changed (GstVaapiDecoder * decoder,
     const GstVideoCodecState * codec_state, gpointer user_data)
@@ -213,9 +218,14 @@ gst_vaapidecode_ensure_allowed_srcpad_caps (GstVaapiDecode * decode)
 
   /* Create VA caps */
   out_caps = gst_caps_from_string (GST_VAAPI_MAKE_SURFACE_CAPS);
-  out_caps = gst_caps_make_writable (out_caps);
+  if (!out_caps) {
+    GST_WARNING_OBJECT (decode, "failed to create VA/GL source caps");
+    return FALSE;
+  }
 #if (USE_GLX || USE_EGL)
-  if (gst_vaapi_display_has_opengl (GST_VAAPI_PLUGIN_BASE_DISPLAY (decode))) {
+  if (!GST_VAAPI_PLUGIN_BASE_SRC_PAD_CAN_DMABUF (decode) &&
+      gst_vaapi_display_has_opengl (GST_VAAPI_PLUGIN_BASE_DISPLAY (decode))) {
+    out_caps = gst_caps_make_writable (out_caps);
     gst_caps_append (out_caps,
         gst_caps_from_string (GST_VAAPI_MAKE_GLTEXUPLOAD_CAPS));
   }
@@ -229,6 +239,7 @@ gst_vaapidecode_ensure_allowed_srcpad_caps (GstVaapiDecode * decode)
     return FALSE;
   }
 
+  out_caps = gst_caps_make_writable (out_caps);
   gst_caps_append (out_caps, gst_caps_copy (raw_caps));
   decode->allowed_srcpad_caps = out_caps;
 
@@ -461,20 +472,33 @@ gst_vaapidecode_negotiate (GstVaapiDecode * decode)
   GstVideoDecoder *const vdec = GST_VIDEO_DECODER (decode);
   GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (vdec);
 
-  GST_DEBUG_OBJECT (decode, "Input codec state changed, doing renegotiation");
+  GST_DEBUG_OBJECT (decode, "input codec state changed: renegotiating");
 
+  GST_VIDEO_DECODER_STREAM_LOCK (vdec);
   if (!gst_vaapi_plugin_base_set_caps (plugin, decode->sinkpad_caps, NULL))
     return FALSE;
   if (!gst_vaapidecode_update_src_caps (decode))
     return FALSE;
-  if (!gst_video_decoder_negotiate (vdec))
-    return FALSE;
   if (!gst_vaapi_plugin_base_set_caps (plugin, NULL, decode->srcpad_caps))
     return FALSE;
+  GST_VIDEO_DECODER_STREAM_UNLOCK (vdec);
+
+  if (!gst_video_decoder_negotiate (vdec))
+    return FALSE;
 
   return TRUE;
 }
 
+static gboolean
+is_src_allocator_dmabuf (GstVaapiDecode * decode)
+{
+  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (decode);
+
+  if (!GST_VAAPI_PLUGIN_BASE_SRC_PAD_CAN_DMABUF (plugin))
+    return FALSE;
+  return gst_vaapi_is_dmabuf_allocator (plugin->srcpad_allocator);
+}
+
 static GstFlowReturn
 gst_vaapidecode_push_decoded_frame (GstVideoDecoder * vdec,
     GstVideoCodecFrame * out_frame)
@@ -485,6 +509,8 @@ gst_vaapidecode_push_decoded_frame (GstVideoDecoder * vdec,
   GstFlowReturn ret;
   const GstVaapiRectangle *crop_rect;
   GstVaapiVideoMeta *meta;
+  GstBufferPoolAcquireParams *params = NULL;
+  GstVaapiVideoBufferPoolAcquireParams vaapi_params = { {0,}, };
   guint flags, out_flags = 0;
   gboolean alloc_renegotiate, caps_renegotiate;
 
@@ -506,8 +532,9 @@ gst_vaapidecode_push_decoded_frame (GstVideoDecoder * vdec,
     caps_renegotiate = is_display_resolution_changed (decode, crop_rect);
 
     if (gst_pad_needs_reconfigure (GST_VIDEO_DECODER_SRC_PAD (vdec))
-        || alloc_renegotiate || caps_renegotiate) {
+        || alloc_renegotiate || caps_renegotiate || decode->do_renego) {
 
+      g_atomic_int_set (&decode->do_renego, FALSE);
       if (!gst_vaapidecode_negotiate (decode))
         return GST_FLOW_ERROR;
     }
@@ -515,14 +542,26 @@ gst_vaapidecode_push_decoded_frame (GstVideoDecoder * vdec,
     gst_vaapi_surface_proxy_set_destroy_notify (proxy,
         (GDestroyNotify) gst_vaapidecode_release, gst_object_ref (decode));
 
-    ret = gst_video_decoder_allocate_output_frame (vdec, out_frame);
+    if (is_src_allocator_dmabuf (decode)) {
+      vaapi_params.proxy = gst_vaapi_surface_proxy_ref (proxy);
+      params = (GstBufferPoolAcquireParams *) & vaapi_params;
+    }
+
+    ret = gst_video_decoder_allocate_output_frame_with_params (vdec, out_frame,
+        params);
+    if (params)
+      gst_vaapi_surface_proxy_unref (vaapi_params.proxy);
     if (ret != GST_FLOW_OK)
       goto error_create_buffer;
 
-    meta = gst_buffer_get_vaapi_video_meta (out_frame->output_buffer);
-    if (!meta)
-      goto error_get_meta;
-    gst_vaapi_video_meta_set_surface_proxy (meta, proxy);
+    /* if not dmabuf is negotiated set the vaapi video meta in the
+     * proxy */
+    if (!params) {
+      meta = gst_buffer_get_vaapi_video_meta (out_frame->output_buffer);
+      if (!meta)
+        goto error_get_meta;
+      gst_vaapi_video_meta_set_surface_proxy (meta, proxy);
+    }
 
     flags = gst_vaapi_surface_proxy_get_flags (proxy);
     if (flags & GST_VAAPI_SURFACE_PROXY_FLAG_CORRUPTED)
@@ -542,17 +581,6 @@ gst_vaapidecode_push_decoded_frame (GstVideoDecoder * vdec,
       GST_BUFFER_FLAG_SET (out_frame->output_buffer,
           GST_VIDEO_BUFFER_FLAG_FIRST_IN_BUNDLE);
     }
-
-    if (crop_rect) {
-      GstVideoCropMeta *const crop_meta =
-          gst_buffer_add_video_crop_meta (out_frame->output_buffer);
-      if (crop_meta) {
-        crop_meta->x = crop_rect->x;
-        crop_meta->y = crop_rect->y;
-        crop_meta->width = crop_rect->width;
-        crop_meta->height = crop_rect->height;
-      }
-    }
 #if (USE_GLX || USE_EGL)
     if (decode->has_texture_upload_meta)
       gst_buffer_ensure_texture_upload_meta (out_frame->output_buffer);
@@ -832,7 +860,7 @@ gst_vaapidecode_create (GstVaapiDecode * decode, GstCaps * caps)
         }
       }
       break;
-#if USE_HEVC_DECODER
+#if USE_H265_DECODER
     case GST_VAAPI_CODEC_H265:
       decode->decoder = gst_vaapi_decoder_h265_new (dpy, caps);
 
@@ -884,7 +912,6 @@ gst_vaapidecode_create (GstVaapiDecode * decode, GstCaps * caps)
   gst_vaapi_decoder_set_codec_state_changed_func (decode->decoder,
       gst_vaapi_decoder_state_changed, decode);
 
-  decode->decoder_caps = gst_caps_ref (caps);
   return TRUE;
 }
 
@@ -921,29 +948,28 @@ gst_vaapidecode_destroy (GstVaapiDecode * decode)
   gst_vaapidecode_purge (decode);
 
   gst_vaapi_decoder_replace (&decode->decoder, NULL);
-  gst_caps_replace (&decode->decoder_caps, NULL);
 
   gst_vaapidecode_release (gst_object_ref (decode));
 }
 
 static gboolean
-gst_vaapidecode_reset_full (GstVaapiDecode * decode, GstCaps * caps,
-    gboolean hard)
+gst_vaapidecode_reset (GstVaapiDecode * decode, GstCaps * caps,
+    gboolean force_reset)
 {
-  GstVaapiCodec codec;
-
   /* Reset tracked frame size */
   decode->current_frame_size = 0;
 
-  if (!hard && decode->decoder && decode->decoder_caps) {
-    if (gst_caps_is_always_compatible (caps, decode->decoder_caps))
-      return TRUE;
-    codec = gst_vaapi_codec_from_caps (caps);
-    if (codec == gst_vaapi_decoder_get_codec (decode->decoder))
-      return TRUE;
+  if (decode->decoder) {
+    if (!gst_caps_is_equal (caps, gst_vaapi_decoder_get_caps (decode->decoder))) {
+      if (gst_vaapi_decoder_update_caps (decode->decoder, caps)) {
+        g_atomic_int_set (&decode->do_renego, TRUE);
+        if (!force_reset)
+          return TRUE;
+      }
+    }
+    return gst_vaapi_decoder_reset (decode->decoder);
   }
 
-  gst_vaapidecode_destroy (decode);
   return gst_vaapidecode_create (decode, caps);
 }
 
@@ -952,8 +978,6 @@ gst_vaapidecode_finalize (GObject * object)
 {
   GstVaapiDecode *const decode = GST_VAAPIDECODE (object);
 
-  gst_caps_replace (&decode->allowed_sinkpad_caps, NULL);
-
   g_cond_clear (&decode->surface_ready);
   g_mutex_clear (&decode->surface_ready_mutex);
 
@@ -983,6 +1007,7 @@ gst_vaapidecode_close (GstVideoDecoder * vdec)
 
   gst_vaapidecode_destroy (decode);
   gst_caps_replace (&decode->allowed_srcpad_caps, NULL);
+  gst_caps_replace (&decode->allowed_sinkpad_caps, NULL);
   gst_vaapi_plugin_base_close (GST_VAAPI_PLUGIN_BASE (decode));
   return TRUE;
 }
@@ -1016,7 +1041,6 @@ gst_vaapidecode_stop (GstVideoDecoder * vdec)
   gst_vaapidecode_purge (decode);
   gst_vaapi_decode_input_state_replace (decode, NULL);
   gst_vaapi_decoder_replace (&decode->decoder, NULL);
-  gst_caps_replace (&decode->decoder_caps, NULL);
   gst_caps_replace (&decode->sinkpad_caps, NULL);
   gst_caps_replace (&decode->srcpad_caps, NULL);
   return TRUE;
@@ -1026,8 +1050,6 @@ static gboolean
 gst_vaapidecode_flush (GstVideoDecoder * vdec)
 {
   GstVaapiDecode *const decode = GST_VAAPIDECODE (vdec);
-  gboolean reverse;
-
   if (!decode->decoder)
     return FALSE;
 
@@ -1035,13 +1057,9 @@ gst_vaapidecode_flush (GstVideoDecoder * vdec)
 
   gst_vaapidecode_purge (decode);
 
-  /* in reverse playback we cannot destroy the decoder at flush, since
-   * it will lost the parsing state */
-  reverse = decode->in_segment.rate < 0;
-
-  /* There could be issues if we avoid the reset_full() while doing
+  /* There could be issues if we avoid the reset() while doing
    * seeking: we have to reset the internal state */
-  return gst_vaapidecode_reset_full (decode, decode->sinkpad_caps, !reverse);
+  return gst_vaapidecode_reset (decode, decode->sinkpad_caps, TRUE);
 }
 
 static gboolean
@@ -1056,7 +1074,7 @@ gst_vaapidecode_set_format (GstVideoDecoder * vdec, GstVideoCodecState * state)
     return FALSE;
   if (!gst_vaapi_plugin_base_set_caps (plugin, decode->sinkpad_caps, NULL))
     return FALSE;
-  if (!gst_vaapidecode_reset_full (decode, decode->sinkpad_caps, TRUE))
+  if (!gst_vaapidecode_reset (decode, decode->sinkpad_caps, FALSE))
     return FALSE;
 
   return TRUE;
@@ -1208,12 +1226,11 @@ static gboolean
 gst_vaapidecode_sink_query (GstVideoDecoder * vdec, GstQuery * query)
 {
   gboolean ret = TRUE;
-  GstVaapiDecode *const decode = GST_VAAPIDECODE (vdec);
-  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (decode);
+  GstElement *const element = GST_ELEMENT (vdec);
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_CONTEXT:{
-      ret = gst_vaapi_handle_context_query (query, plugin->display);
+      ret = gst_vaapi_handle_context_query (element, query);
       break;
     }
     default:{
@@ -1229,8 +1246,7 @@ static gboolean
 gst_vaapidecode_src_query (GstVideoDecoder * vdec, GstQuery * query)
 {
   gboolean ret = TRUE;
-  GstVaapiDecode *const decode = GST_VAAPIDECODE (vdec);
-  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (decode);
+  GstElement *const element = GST_ELEMENT (vdec);
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_CAPS:{
@@ -1251,7 +1267,7 @@ gst_vaapidecode_src_query (GstVideoDecoder * vdec, GstQuery * query)
       break;
     }
     case GST_QUERY_CONTEXT:{
-      ret = gst_vaapi_handle_context_query (query, plugin->display);
+      ret = gst_vaapi_handle_context_query (element, query);
       break;
     }
     default:{
