@@ -30,10 +30,15 @@
 #include "gstvaapivideocontext.h"
 #include "gstvaapivideometa.h"
 #include "gstvaapivideobufferpool.h"
+#if USE_GST_GL_HELPERS
+# include <gst/gl/gl.h>
+#endif
 
 /* Default debug category is from the subclass */
 #define GST_CAT_DEFAULT (plugin->debug_category)
 
+#define BUFFER_POOL_SINK_MIN_BUFFERS 2
+
 /* GstVideoContext interface */
 static void
 plugin_set_display (GstVaapiPluginBase * plugin, GstVaapiDisplay * display)
@@ -46,7 +51,7 @@ plugin_set_display (GstVaapiPluginBase * plugin, GstVaapiDisplay * display)
         display_name, plugin->display_name);
     gst_vaapi_display_replace (&plugin->display, NULL);
   } else {
-    GST_INFO_OBJECT (plugin, "set display %p", display);
+    GST_INFO_OBJECT (plugin, "set display %" GST_PTR_FORMAT, display);
     gst_vaapi_display_replace (&plugin->display, display);
     plugin->display_type = gst_vaapi_display_get_display_type (display);
     gst_vaapi_plugin_base_set_display_name (plugin, display_name);
@@ -73,6 +78,12 @@ gst_vaapi_plugin_base_set_context (GstVaapiPluginBase * plugin,
 
   if (gst_vaapi_video_context_get_display (context, &display))
     plugin_set_display (plugin, display);
+
+#if USE_GST_GL_HELPERS
+  gst_gl_handle_set_context (GST_ELEMENT_CAST (plugin), context,
+      (GstGLDisplay **) & plugin->gl_display,
+      (GstGLContext **) & plugin->gl_other_context);
+#endif
 }
 
 void
@@ -213,17 +224,6 @@ plugin_reset_texture_map (GstVaapiPluginBase * plugin)
     gst_vaapi_display_reset_texture_map (plugin->display);
 }
 
-static void
-gst_vaapi_plugin_base_find_gl_context (GstVaapiPluginBase * plugin)
-{
-  GstObject *gl_context;
-
-  if (!gst_vaapi_find_gl_local_context (GST_ELEMENT_CAST (plugin), &gl_context))
-    return;
-  gst_vaapi_plugin_base_set_gl_context (plugin, gl_context);
-  gst_object_unref (gl_context);
-}
-
 void
 gst_vaapi_plugin_base_class_init (GstVaapiPluginBaseClass * klass)
 {
@@ -247,6 +247,9 @@ gst_vaapi_plugin_base_init (GstVaapiPluginBase * plugin,
   if (!(GST_OBJECT_FLAGS (plugin) & GST_ELEMENT_FLAG_SINK))
     plugin->srcpad = gst_element_get_static_pad (GST_ELEMENT (plugin), "src");
   gst_video_info_init (&plugin->srcpad_info);
+
+  plugin->enable_direct_rendering =
+      (g_getenv ("GST_VAAPI_ENABLE_DIRECT_RENDERING") != NULL);
 }
 
 void
@@ -290,6 +293,8 @@ gst_vaapi_plugin_base_close (GstVaapiPluginBase * plugin)
 
   gst_vaapi_display_replace (&plugin->display, NULL);
   gst_object_replace (&plugin->gl_context, NULL);
+  gst_object_replace (&plugin->gl_display, NULL);
+  gst_object_replace (&plugin->gl_other_context, NULL);
 
   gst_caps_replace (&plugin->sinkpad_caps, NULL);
   gst_video_info_init (&plugin->sinkpad_info);
@@ -384,10 +389,6 @@ gst_vaapi_plugin_base_ensure_display (GstVaapiPluginBase * plugin)
     return TRUE;
   gst_vaapi_display_replace (&plugin->display, NULL);
 
-  /* Query for a local GstGL context. If it's found, it will be used
-   * to create the VA display */
-  gst_vaapi_plugin_base_find_gl_context (plugin);
-
   if (!gst_vaapi_ensure_display (GST_ELEMENT (plugin),
           plugin->display_type_req))
     return FALSE;
@@ -459,32 +460,6 @@ has_dmabuf_capable_peer (GstVaapiPluginBase * plugin, GstPad * pad)
   return is_dmabuf_capable;
 }
 
-static gboolean
-set_dmabuf_allocator (GstVaapiPluginBase * plugin, GstBufferPool * pool,
-    GstCaps * caps)
-{
-  GstStructure *config;
-  GstAllocator *allocator;
-  GstVideoInfo vi;
-
-  if (!gst_video_info_from_caps (&vi, caps))
-    return FALSE;
-  allocator = gst_vaapi_dmabuf_allocator_new (plugin->display, &vi,
-      GST_VAAPI_SURFACE_ALLOC_FLAG_LINEAR_STORAGE);
-  if (!allocator)
-    return FALSE;
-
-  config = gst_buffer_pool_get_config (pool);
-  gst_buffer_pool_config_set_allocator (config, allocator, NULL);
-  if (!gst_buffer_pool_set_config (pool, config))
-    return FALSE;
-
-  if (plugin->sinkpad_allocator)
-    gst_object_unref (plugin->sinkpad_allocator);
-  plugin->sinkpad_allocator = allocator;
-  return TRUE;
-}
-
 static gboolean
 gst_vaapi_buffer_pool_caps_is_equal (GstBufferPool * pool, GstCaps * newcaps)
 {
@@ -519,31 +494,147 @@ reset_allocator (GstAllocator * allocator, GstVideoInfo * vinfo)
 }
 
 static gboolean
-ensure_sinkpad_allocator (GstVaapiPluginBase * plugin, GstVideoInfo * vinfo)
+ensure_sinkpad_allocator (GstVaapiPluginBase * plugin, GstCaps * caps,
+    guint * size)
 {
-  if (!reset_allocator (plugin->sinkpad_allocator, vinfo))
+  GstVideoInfo vinfo;
+  GstVaapiImageUsageFlags usage_flag =
+      GST_VAAPI_IMAGE_USAGE_FLAG_NATIVE_FORMATS;
+
+  if (!gst_video_info_from_caps (&vinfo, caps))
+    goto error_invalid_caps;
+  gst_video_info_force_nv12_if_encoded (&vinfo);
+  *size = GST_VIDEO_INFO_SIZE (&vinfo);
+
+  if (!reset_allocator (plugin->sinkpad_allocator, &vinfo))
     return TRUE;
 
   if (has_dmabuf_capable_peer (plugin, plugin->sinkpad)) {
     plugin->sinkpad_allocator =
-        gst_vaapi_dmabuf_allocator_new (plugin->display, vinfo,
-        GST_VAAPI_SURFACE_ALLOC_FLAG_LINEAR_STORAGE);
-  } else {
-    plugin->sinkpad_allocator =
-        gst_vaapi_video_allocator_new (plugin->display, vinfo, 0);
+        gst_vaapi_dmabuf_allocator_new (plugin->display, &vinfo,
+        GST_VAAPI_SURFACE_ALLOC_FLAG_LINEAR_STORAGE, GST_PAD_SINK);
+    goto bail;
   }
-  return plugin->sinkpad_allocator != NULL;
+
+  /* enable direct upload if upstream requests raw video */
+  if (gst_caps_is_video_raw (caps)) {
+    usage_flag = GST_VAAPI_IMAGE_USAGE_FLAG_DIRECT_UPLOAD;
+    GST_INFO_OBJECT (plugin, "enabling direct upload in sink allocator");
+  }
+  plugin->sinkpad_allocator =
+      gst_vaapi_video_allocator_new (plugin->display, &vinfo, 0, usage_flag);
+
+bail:
+  if (!plugin->sinkpad_allocator)
+    goto error_create_allocator;
+  return TRUE;
+
+  /* ERRORS */
+error_invalid_caps:
+  {
+    GST_ERROR_OBJECT (plugin, "invalid caps %" GST_PTR_FORMAT, caps);
+    return FALSE;
+  }
+error_create_allocator:
+  {
+    GST_ERROR_OBJECT (plugin, "failed to create sink pad's allocator");
+    return FALSE;
+  }
+}
+
+static inline guint
+get_dmabuf_surface_allocation_flags (void)
+{
+  /* @FIXME: fetch the real devices ids */
+  /* Pair vendor/device identifies an unique physical device. */
+  guint va_vendor_id = 0x00;
+  guint va_device_id = 0x00;
+  guint gl_vendor_id = 0x00;
+  guint gl_device_id = 0x00;
+
+  /* Requires linear memory only if fd export is done on a different
+   * device than the device where the fd is imported. */
+  gboolean same_physical_device = va_vendor_id == gl_vendor_id
+      && va_device_id == gl_device_id;
+
+  if (same_physical_device)
+    return 0;
+  return GST_VAAPI_SURFACE_ALLOC_FLAG_LINEAR_STORAGE;
 }
 
 static gboolean
-ensure_srcpad_allocator (GstVaapiPluginBase * plugin, GstVideoInfo * vinfo)
+ensure_srcpad_allocator (GstVaapiPluginBase * plugin, GstVideoInfo * vinfo,
+    GstCaps * caps)
 {
-  if (!reset_allocator (plugin->srcpad_allocator, vinfo))
+  gboolean different_caps;
+  GstVideoInfo vi;
+  GstVaapiImageUsageFlags usage_flag =
+      GST_VAAPI_IMAGE_USAGE_FLAG_NATIVE_FORMATS;
+
+  /* the received caps are the "allocation caps" which may be
+   * different from the "negotiation caps". In this case, we should
+   * indicate the allocator to store the negotiation caps since they
+   * are the one should be used for frame mapping with GstVideoMeta */
+  different_caps = GST_IS_VIDEO_DECODER (plugin) && plugin->srcpad_caps &&
+      !gst_caps_is_strictly_equal (plugin->srcpad_caps, caps);
+
+  if (different_caps) {
+    vi = plugin->srcpad_info;
+  } else {
+    vi = *vinfo;
+  }
+
+  if (!reset_allocator (plugin->srcpad_allocator, &vi))
     return TRUE;
 
-  plugin->srcpad_allocator =
-      gst_vaapi_video_allocator_new (plugin->display, vinfo, 0);
-  return plugin->srcpad_allocator != NULL;
+  plugin->srcpad_allocator = NULL;
+  if (caps && gst_caps_is_video_raw (caps)) {
+    if (plugin->srcpad_can_dmabuf) {
+      if (GST_IS_VIDEO_DECODER (plugin) || GST_IS_BASE_TRANSFORM (plugin)) {
+        plugin->srcpad_allocator =
+            gst_vaapi_dmabuf_allocator_new (plugin->display, vinfo,
+            get_dmabuf_surface_allocation_flags (), GST_PAD_SRC);
+      }
+    } else if (plugin->enable_direct_rendering) {
+      usage_flag = GST_VAAPI_IMAGE_USAGE_FLAG_DIRECT_RENDER;
+      GST_INFO_OBJECT (plugin, "enabling direct rendering in source allocator");
+    }
+  }
+
+  if (!plugin->srcpad_allocator) {
+    plugin->srcpad_allocator =
+        gst_vaapi_video_allocator_new (plugin->display, vinfo, 0, usage_flag);
+  }
+
+  if (!plugin->srcpad_allocator)
+    goto error_create_allocator;
+
+  if (different_caps) {
+    guint i, flags = 0;
+    const GstVideoInfo *alloc_vi =
+        gst_allocator_get_vaapi_video_info (plugin->srcpad_allocator, &flags);
+    /* update the planes and the size with the allocator image info,
+     * but not the resolution */
+    if (alloc_vi) {
+      for (i = 0; i < GST_VIDEO_INFO_N_PLANES (alloc_vi); i++) {
+        GST_VIDEO_INFO_PLANE_OFFSET (&vi, i) =
+            GST_VIDEO_INFO_PLANE_OFFSET (alloc_vi, i);
+        GST_VIDEO_INFO_PLANE_STRIDE (&vi, i) =
+            GST_VIDEO_INFO_PLANE_STRIDE (alloc_vi, i);
+      }
+      GST_VIDEO_INFO_SIZE (&vi) = GST_VIDEO_INFO_SIZE (alloc_vi);
+      gst_allocator_set_vaapi_video_info (plugin->srcpad_allocator, &vi, flags);
+    }
+  }
+
+  return TRUE;
+
+  /* ERRORS */
+error_create_allocator:
+  {
+    GST_ERROR_OBJECT (plugin, "failed to create src pad's allocator");
+    return FALSE;
+  }
 }
 
 /**
@@ -634,7 +725,6 @@ static gboolean
 ensure_sinkpad_buffer_pool (GstVaapiPluginBase * plugin, GstCaps * caps)
 {
   GstBufferPool *pool;
-  GstVideoInfo vi;
   guint size;
 
   /* video decoders don't use a buffer pool in the sink pad */
@@ -653,39 +743,19 @@ ensure_sinkpad_buffer_pool (GstVaapiPluginBase * plugin, GstCaps * caps)
     plugin->sinkpad_buffer_size = 0;
   }
 
-  if (!gst_video_info_from_caps (&vi, caps))
-    goto error_invalid_caps;
-  gst_video_info_force_nv12_if_encoded (&vi);
+  if (!ensure_sinkpad_allocator (plugin, caps, &size))
+    return FALSE;
 
-  if (!ensure_sinkpad_allocator (plugin, &vi))
-    goto error_create_allocator;
-
-  size = GST_VIDEO_INFO_SIZE (&vi);
-  gst_allocator_get_vaapi_image_size (plugin->sinkpad_allocator, &size);
-  pool = gst_vaapi_plugin_base_create_pool (plugin, caps, size, 0, 0,
+  pool =
+      gst_vaapi_plugin_base_create_pool (plugin, caps, size,
+      BUFFER_POOL_SINK_MIN_BUFFERS, 0,
       GST_VAAPI_VIDEO_BUFFER_POOL_OPTION_VIDEO_META, plugin->sinkpad_allocator);
   if (!pool)
-    goto error_create_pool;
+    return FALSE;
+
   plugin->sinkpad_buffer_pool = pool;
   plugin->sinkpad_buffer_size = size;
   return TRUE;
-
-  /* ERRORS */
-error_invalid_caps:
-  {
-    GST_ERROR_OBJECT (plugin, "invalid caps %" GST_PTR_FORMAT, caps);
-    return FALSE;
-  }
-error_create_allocator:
-  {
-    GST_ERROR_OBJECT (plugin, "failed to create allocator");
-    return FALSE;
-  }
-error_create_pool:
-  {
-    /* error message already sent */
-    return FALSE;
-  }
 }
 
 /**
@@ -704,19 +774,24 @@ gst_vaapi_plugin_base_set_caps (GstVaapiPluginBase * plugin, GstCaps * incaps,
     GstCaps * outcaps)
 {
   if (incaps && incaps != plugin->sinkpad_caps) {
-    g_clear_object (&plugin->sinkpad_allocator);
-    gst_caps_replace (&plugin->sinkpad_caps, incaps);
     if (!gst_video_info_from_caps (&plugin->sinkpad_info, incaps))
       return FALSE;
+    gst_caps_replace (&plugin->sinkpad_caps, incaps);
     plugin->sinkpad_caps_is_raw = !gst_caps_has_vaapi_surface (incaps);
   }
 
   if (outcaps && outcaps != plugin->srcpad_caps) {
-    g_clear_object (&plugin->srcpad_allocator);
-    gst_caps_replace (&plugin->srcpad_caps, outcaps);
-    plugin_reset_texture_map (plugin);
     if (!gst_video_info_from_caps (&plugin->srcpad_info, outcaps))
       return FALSE;
+    if (plugin->srcpad_buffer_pool
+        && !gst_vaapi_buffer_pool_caps_is_equal (plugin->srcpad_buffer_pool,
+            outcaps)) {
+      gst_buffer_pool_set_active (plugin->srcpad_buffer_pool, FALSE);
+      g_clear_object (&plugin->srcpad_buffer_pool);
+      g_clear_object (&plugin->srcpad_allocator);
+      plugin_reset_texture_map (plugin);
+    }
+    gst_caps_replace (&plugin->srcpad_caps, outcaps);
   }
 
   if (!ensure_sinkpad_buffer_pool (plugin, plugin->sinkpad_caps))
@@ -748,12 +823,8 @@ gst_vaapi_plugin_base_propose_allocation (GstVaapiPluginBase * plugin,
     if (!ensure_sinkpad_buffer_pool (plugin, caps))
       return FALSE;
     gst_query_add_allocation_pool (query, plugin->sinkpad_buffer_pool,
-        plugin->sinkpad_buffer_size, 0, 0);
-
-    if (has_dmabuf_capable_peer (plugin, plugin->sinkpad)) {
-      if (!set_dmabuf_allocator (plugin, plugin->sinkpad_buffer_pool, caps))
-        goto error_pool_config;
-    }
+        plugin->sinkpad_buffer_size, BUFFER_POOL_SINK_MIN_BUFFERS, 0);
+    gst_query_add_allocation_param (query, plugin->sinkpad_allocator, NULL);
   }
 
   gst_query_add_allocation_meta (query, GST_VAAPI_VIDEO_META_API_TYPE, NULL);
@@ -766,11 +837,6 @@ error_no_caps:
     GST_INFO_OBJECT (plugin, "no caps specified");
     return FALSE;
   }
-error_pool_config:
-  {
-    GST_ERROR_OBJECT (plugin, "failed to reset buffer pool config");
-    return FALSE;
-  }
 }
 
 /**
@@ -792,7 +858,7 @@ gst_vaapi_plugin_base_decide_allocation (GstVaapiPluginBase * plugin,
   GstBufferPool *pool;
   GstVideoInfo vi;
   guint size, min, max, pool_options;
-  gboolean update_pool = FALSE;
+  gboolean update_pool = FALSE, update_allocator = FALSE;
 #if (USE_GLX || USE_EGL)
   guint idx;
 #endif
@@ -820,7 +886,7 @@ gst_vaapi_plugin_base_decide_allocation (GstVaapiPluginBase * plugin,
 
     gst_query_parse_nth_allocation_meta (query, idx, &params);
     if (params) {
-      if (gst_structure_get (params, "gst.gl.GstGLContext", GST_GL_TYPE_CONTEXT,
+      if (gst_structure_get (params, "gst.gl.GstGLContext", GST_TYPE_GL_CONTEXT,
               &gl_context, NULL) && gl_context) {
         gst_vaapi_plugin_base_set_gl_context (plugin, gl_context);
         gst_object_unref (gl_context);
@@ -840,6 +906,9 @@ gst_vaapi_plugin_base_decide_allocation (GstVaapiPluginBase * plugin,
     goto error_invalid_caps;
   gst_video_info_force_nv12_if_encoded (&vi);
 
+  if (gst_query_get_n_allocation_params (query) > 0)
+    update_allocator = TRUE;
+
   if (gst_query_get_n_allocation_pools (query) > 0) {
     gst_query_parse_nth_allocation_pool (query, 0, &pool, &size, &min, &max);
     update_pool = TRUE;
@@ -866,14 +935,12 @@ gst_vaapi_plugin_base_decide_allocation (GstVaapiPluginBase * plugin,
   }
 
   if (!pool) {
-    if (!ensure_srcpad_allocator (plugin, &vi))
-      goto error_create_allocator;
-    /* Update video size with allocator's image size */
-    gst_allocator_get_vaapi_image_size (plugin->srcpad_allocator, &size);
+    if (!ensure_srcpad_allocator (plugin, &vi, caps))
+      goto error;
     pool = gst_vaapi_plugin_base_create_pool (plugin, caps, size, min, max,
         pool_options, plugin->srcpad_allocator);
     if (!pool)
-      goto error_create_pool;
+      goto error;
   }
 
   if (update_pool)
@@ -881,6 +948,12 @@ gst_vaapi_plugin_base_decide_allocation (GstVaapiPluginBase * plugin,
   else
     gst_query_add_allocation_pool (query, pool, size, min, max);
 
+  if (update_allocator)
+    gst_query_set_nth_allocation_param (query, 0, plugin->srcpad_allocator,
+        NULL);
+  else
+    gst_query_add_allocation_param (query, plugin->srcpad_allocator, NULL);
+
   g_clear_object (&plugin->srcpad_buffer_pool);
   plugin->srcpad_buffer_pool = pool;
   return TRUE;
@@ -902,12 +975,7 @@ error_ensure_display:
         plugin->display_type_req);
     return FALSE;
   }
-error_create_allocator:
-  {
-    GST_ERROR_OBJECT (plugin, "failed to create allocator");
-    return FALSE;
-  }
-error_create_pool:
+error:
   {
     /* error message already sent */
     return FALSE;
@@ -951,7 +1019,8 @@ gst_vaapi_plugin_base_get_input_buffer (GstVaapiPluginBase * plugin,
   if (!plugin->sinkpad_buffer_pool)
     goto error_no_pool;
 
-  if (!gst_buffer_pool_set_active (plugin->sinkpad_buffer_pool, TRUE))
+  if (!gst_buffer_pool_is_active (plugin->sinkpad_buffer_pool) &&
+      !gst_buffer_pool_set_active (plugin->sinkpad_buffer_pool, TRUE))
     goto error_active_pool;
 
   outbuf = NULL;
@@ -1068,8 +1137,14 @@ gst_vaapi_plugin_base_set_gl_context (GstVaapiPluginBase * plugin,
       display_type = GST_VAAPI_DISPLAY_TYPE_GLX;
       break;
 #endif
-#if USE_EGL
     case GST_GL_PLATFORM_EGL:
+#if VA_CHECK_VERSION (0,36,0) && USE_GST_GL_HELPERS
+      plugin->srcpad_can_dmabuf =
+          (!(gst_gl_context_get_gl_api (gl_context) & GST_GL_API_GLES1)
+          && gst_gl_context_check_feature (gl_context,
+              "EGL_EXT_image_dma_buf_import"));
+#endif
+#if USE_EGL
       display_type = GST_VAAPI_DISPLAY_TYPE_EGL;
       break;
 #endif
@@ -1082,6 +1157,56 @@ gst_vaapi_plugin_base_set_gl_context (GstVaapiPluginBase * plugin,
 #endif
 }
 
+/**
+ * gst_vaapi_plugin_base_create_gl_context:
+ * @plugin: a #GstVaapiPluginBase
+ *
+ * It queries downstream and upstream for a #GstGLDisplay and a other
+ * #GstGLContext. If not found, a new #GstGLDisplay and #GstGLContext
+ * are created, if it is possible.
+ *
+ * Returns: (transfer full) a new created #GstGLContext or %NULL
+ **/
+GstObject *
+gst_vaapi_plugin_base_create_gl_context (GstVaapiPluginBase * plugin)
+{
+#if USE_GST_GL_HELPERS
+  GstGLContext *gl_other_context, *gl_context = NULL;
+  GstGLDisplay *gl_display;
+
+  gst_gl_ensure_element_data (plugin, (GstGLDisplay **) & plugin->gl_display,
+      (GstGLContext **) & plugin->gl_other_context);
+
+  gl_display = (GstGLDisplay *) plugin->gl_display;
+  if (!gl_display ||
+      gst_gl_display_get_handle_type (gl_display) == GST_GL_DISPLAY_TYPE_ANY) {
+    gst_object_replace (&plugin->gl_display, NULL);
+    gst_object_replace (&plugin->gl_other_context, NULL);
+    return NULL;
+  }
+  gl_other_context = (GstGLContext *) plugin->gl_other_context;
+
+  GST_INFO_OBJECT (plugin, "creating a new GstGL context");
+
+  GST_OBJECT_LOCK (gl_display);
+  do {
+    if (gl_context)
+      gst_object_unref (gl_context);
+    gl_context = gst_gl_display_get_gl_context_for_thread (gl_display, NULL);
+    if (!gl_context) {
+      if (!gst_gl_display_create_context (gl_display, gl_other_context,
+              &gl_context, NULL))
+        break;
+    }
+  } while (!gst_gl_display_add_context (gl_display, gl_context));
+  GST_OBJECT_UNLOCK (gl_display);
+
+  return GST_OBJECT_CAST (gl_context);
+#else
+  return NULL;
+#endif
+}
+
 static gboolean
 ensure_allowed_raw_caps (GstVaapiPluginBase * plugin)
 {
