@@ -49,7 +49,11 @@
 #define GST_PLUGIN_DESC "A VA-API video postprocessing filter"
 
 GST_DEBUG_CATEGORY_STATIC (gst_debug_vaapipostproc);
+#ifndef GST_DISABLE_GST_DEBUG
 #define GST_CAT_DEFAULT gst_debug_vaapipostproc
+#else
+#define GST_CAT_DEFAULT NULL
+#endif
 
 /* Default templates */
 /* *INDENT-OFF* */
@@ -362,7 +366,8 @@ create_output_buffer (GstVaapiPostproc * postproc)
 
   g_return_val_if_fail (pool != NULL, NULL);
 
-  if (!gst_buffer_pool_set_active (pool, TRUE))
+  if (!gst_buffer_pool_is_active (pool) &&
+      !gst_buffer_pool_set_active (pool, TRUE))
     goto error_activate_pool;
 
   outbuf = NULL;
@@ -1064,9 +1069,9 @@ expand_allowed_srcpad_caps (GstVaapiPostproc * postproc, GstCaps * caps)
   }
   g_value_unset (&value);
 
-  /* remove GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META if
-   * the backend doesn't support opengl */
-  if (!gst_vaapi_display_has_opengl (GST_VAAPI_PLUGIN_BASE_DISPLAY (postproc))
+  if ((GST_VAAPI_PLUGIN_BASE_SRC_PAD_CAN_DMABUF (postproc)
+          || !gst_vaapi_display_has_opengl (GST_VAAPI_PLUGIN_BASE_DISPLAY
+              (postproc)))
       && gl_upload_meta_idx > -1) {
     gst_caps_remove_structure (caps, gl_upload_meta_idx);
   }
@@ -1345,11 +1350,11 @@ gst_vaapipostproc_query (GstBaseTransform * trans,
     GstPadDirection direction, GstQuery * query)
 {
   GstVaapiPostproc *const postproc = GST_VAAPIPOSTPROC (trans);
+  GstElement *const element = GST_ELEMENT (trans);
 
   if (GST_QUERY_TYPE (query) == GST_QUERY_CONTEXT) {
-    if (gst_vaapi_handle_context_query (query,
-            GST_VAAPI_PLUGIN_BASE_DISPLAY (postproc))) {
-      GST_DEBUG_OBJECT (postproc, "sharing display %p",
+    if (gst_vaapi_handle_context_query (element, query)) {
+      GST_DEBUG_OBJECT (postproc, "sharing display %" GST_PTR_FORMAT,
           GST_VAAPI_PLUGIN_BASE_DISPLAY (postproc));
       return TRUE;
     }
